import abc
import collections
from datetime import datetime, date, time
from decimal import Decimal

from django.db.models import Model
from django.http import HttpResponse
from django.utils import simplejson
from django.utils.encoding import smart_unicode

from .paginator import Paginator
from .tools import as_tuple


class AbstractSerializer(object):

    __meta__ = abc.ABCMeta

    def __init__(self, **options):
        self.options = self.init_options(**options)

    @staticmethod
    def init_options(_fields=None, _include=None, _exclude=None, **related):
        options = dict(
            _fields=set(_fields and as_tuple(_fields) or []),
            _include=set(_include and as_tuple(_include) or []),
            _exclude=set(_exclude and as_tuple(_exclude) or []),
        )
        options.update(related)
        return options

    def to_simple(self, value, **options):
        " Simplify object. "

        options = options or self.options

        if isinstance(value, basestring):
            return smart_unicode(value)

        if isinstance(value, Decimal):
            return float(str(value))

        if isinstance(value, (datetime, date, time)):
            return self.to_simple_datetime(value)

        if isinstance(value, dict):
            return dict((k, self.to_simple(v, **options)) for k, v in value.iteritems())

        if isinstance(value, Paginator):
            return dict(
                count=value.count,
                page=value.page.number,
                next=value.next,
                prev=value.previous,
                resources=self.to_simple(value.resources, **options)
            )

        if isinstance(value, collections.Iterable):
            return [self.to_simple(o, **options) for o in value]

        if isinstance(value, Model):
            return self.to_simple_model(value, **options)

        return value

    @staticmethod
    def to_simple_datetime(value):
        result = value.isoformat()
        if isinstance(value, datetime):
            if value.microsecond:
                result = result[:23] + result[26:]
            if result.endswith('+00:00'):
                result = result[:-6] + 'Z'
        elif isinstance(value, time) and value.microsecond:
            result = result[:12]
        return result

    def to_simple_model(self, value, **options):

        result = dict(
            model=smart_unicode(value._meta),
            pk=smart_unicode(value._get_pk_val(), strings_only=True),
            fields=dict(),
        )

        default_fields = set([field.name for field in value._meta.fields + value._meta.many_to_many if field.serialize])
        serialized_fields = (default_fields | options['_include']) - options['_exclude']
        for fname in options['_fields'] or serialized_fields:

            if options.get(fname):
                result['fields'][fname] = self.to_simple(
                    getattr(value, fname),
                    **self.init_options(**options.get(fname)))
                continue

            if fname in default_fields:
                field = value._meta.get_field(fname)
                result['fields'][fname] = self.to_simple(field.value_from_object(value))
                continue

            result['fields'][fname] = self.to_simple(getattr(value, fname, None))

        return result

    @abc.abstractmethod
    def serialize(self, value):
        raise NotImplementedError


class BaseSerializer(AbstractSerializer):

    def serialize(self, value):

        if isinstance(value, HttpResponse):
            return value.content

        return self.to_simple(value)


class JSONSerializer(BaseSerializer):

    def serialize(self, value):

        if isinstance(value, HttpResponse):
            return value.content

        return simplejson.dumps(self.to_simple(value))


class XMLSerializer(BaseSerializer):

    def serialize(self, value):

        if isinstance(value, HttpResponse):
            return value.content

        return ''.join(s for s in self._dumps(self.to_simple(value)))

    def _dumps(self, value):
        tag = it = None

        if isinstance(value, list):
            tag = 'items'
            it = iter(value)

        elif isinstance(value, dict) and 'model' in value:
            tag = value.get('model').split('.')[1]
            it = value.iteritems()

        elif isinstance(value, dict):
            it = value.iteritems()

        elif isinstance(value, tuple):
            tag = str(value[0])
            it = (i for i in value[1:])

        else:
            yield str(value)

        if tag:
            yield "<%s>" % tag

        if it:
            try:
                while True:
                    v = next(it)
                    yield ''.join(self._dumps(v))
            except StopIteration:
                yield ''

        if tag:
            yield "</%s>" % tag


# lint_ignore=W901,R0911



from logging import getLogger

from django.core.exceptions import FieldError
from django.db.models import get_model, Model
from django.db.models.sql.constants import LOOKUP_SEP
from django.http import HttpResponse

from ..forms import PartitialForm
from ..settings import LIMIT_PER_PAGE
from ..utils import status, MetaOptions, UpdatedList
from ..utils.exceptions import HttpError
from ..utils.paginator import Paginator
from ..utils.tools import as_tuple


logger = getLogger('django.request')


class HandlerMeta(type):

    def __new__(mcs, name, bases, params):

        params['meta'] = params.get('meta', MetaOptions())
        cls = super(HandlerMeta, mcs).__new__(mcs, name, bases, params)

        # Create model from string
        if isinstance(cls.model, basestring):
            assert '.' in cls.model, ("'model_class' must be either a model"
                                      " or a model name in the format"
                                      " app_label.model_name")
            cls.model = get_model(*cls.model.split("."))

        # Check meta.name and queryset
        if cls.model:
            assert issubclass(cls.model, Model), "'model' attribute must be subclass of Model "
            cls.meta.name = cls.model._meta.module_name
            cls.meta.model_fields = set(f.name for f in cls.model._meta.fields)
            if cls.queryset is None:
                cls.queryset = cls.model.objects.all()

        # Create form if not exist
        if cls.model and not cls.form:
            class DynForm(PartitialForm):
                class Meta():
                    model = cls.model
                    fields = cls.form_fields
                    exclude = cls.form_exclude
            cls.form = DynForm

        return cls


class HandlerMixin(object):

    __metaclass__ = HandlerMeta

    limit_per_page = LIMIT_PER_PAGE
    model = None
    queryset = None
    form = None
    form_fields = None
    form_exclude = None
    callmap = {'GET': 'get', 'POST': 'post',
               'PUT': 'put', 'DELETE': 'delete',
               'PATCH': 'patch', 'OPTIONS': 'options',
               'HEAD': 'head'}

    def __init__(self, *args, **kwargs):
        " Copy self queryset for prevent query caching. "

        super(HandlerMixin, self).__init__(*args, **kwargs)

        if not self.queryset is None:
            self.queryset = self.queryset.all()

    @staticmethod
    def head(*args, **kwargs):
        " Default HEAD method. "

        return HttpResponse()

    def get(self, request, **resources):
        " Default GET method. Return instanse (collection) by model. "

        instance = resources.get(self.meta.name)
        if not instance is None:
            return instance

        return self.paginate(request, self.get_collection(request, **resources))

    def post(self, request, **resources):
        " Default POST method. Uses self form. "

        form = self.form(request.data, **resources)
        if form.is_valid():
            return form.save()

        raise HttpError(
            form.errors.as_text(), status=status.HTTP_400_BAD_REQUEST)

    def put(self, request, **resources):
        " Default PUT method. Uses self form. Allow bulk update. "

        resource = resources.get(self.meta.name)
        if not resource:
            raise HttpError("Resource not found.", status=status.HTTP_404_NOT_FOUND)

        updated = UpdatedList()
        for o in as_tuple(resource):
            form = self.form(
                data=request.data, instance=o, **resources)

            if not form.is_valid():
                raise HttpError(
                    form.errors.as_text(), status=status.HTTP_400_BAD_REQUEST)

            updated.append(form.save())

        return updated if len(updated) > 1 else updated[-1]

    def delete(self, request, **resources):
        " Default DELETE method. Allow bulk delete. "

        resource = resources.get(self.meta.name)
        if not resource:
            raise HttpError("Bad request", status=status.HTTP_404_NOT_FOUND)

        for o in as_tuple(resource):
            o.delete()

        return HttpResponse("")

    def patch(self, request, **resources):
        " Default PATCH method. Do nothing. "

        pass

    @staticmethod
    def options(request, **resources):
        " Default OPTIONS method. Always response OK. "

        return HttpResponse("OK")

    def get_collection(self, request, **resources):
        " Get filters and return filtered result. "

        if self.queryset is None:
            return None

        # Make filters from URL variables or resources
        default_filters = self.get_default_filters(**resources)
        filters = self.get_filters(request, **resources)
        filters.update(default_filters)

        # Get collection by queryset
        qs = self.queryset
        for key, (value, exclude) in filters.items():
            try:
                if exclude:
                    qs = qs.exclude(**{key: value})

                else:
                    qs = qs.filter(**{key: value})
            except FieldError, e:
                logger.warning(e)

        return qs

    def get_default_filters(self, **resources):
        return dict((k, (v, False)) for k, v in resources.items() if k in self.meta.model_fields)

    def get_filters(self, request, **resources):
        " Make filters from GET variables. "

        filters = dict()
        for field in request.GET.iterkeys():
            tokens = field.split(LOOKUP_SEP)
            field_name = tokens[0]

            exclude = False
            if tokens[-1] == 'not':
                exclude = True
                tokens.pop()

            if not field_name in self.meta.model_fields:
                continue

            converter = self.model._meta.get_field(
                field_name).to_python if len(tokens) == 1 else lambda v: v
            value = map(converter, request.GET.getlist(field))

            if len(value) > 1:
                tokens.append('in')
            else:
                value = value.pop()

            filters[LOOKUP_SEP.join(tokens)] = (value, exclude)

        return filters

    def paginate(self, request, qs):
        " Paginate queryset. "

        return Paginator(request, qs, self.limit_per_page)


# pymode:lint_ignore=E1102



from .auth import AuthMixin
from .emitter import EmitterMixin
from .parser import ParserMixin
from .handler import HandlerMixin
from .throttle import ThrottleMixin


assert AuthMixin and EmitterMixin and ParserMixin and HandlerMixin and ThrottleMixin



from ..utils import MetaOptions
from ..utils.parser import FormParser, XMLParser, JSONParser, AbstractParser
from ..utils.tools import as_tuple


class ParserMeta(type):

    def __new__(mcs, name, bases, params):
        params['meta'] = params.get('meta', MetaOptions())
        cls = super(ParserMeta, mcs).__new__(mcs, name, bases, params)
        cls.parsers = as_tuple(cls.parsers)
        cls.meta.default_parser = cls.parsers[0] if cls.parsers else None

        for p in cls.parsers:
            assert issubclass(p, AbstractParser), "Parser must be subclass of AbstractParser"
            cls.meta.parsers_dict[p.media_type] = p

        return cls


class ParserMixin(object):

    __metaclass__ = ParserMeta

    parsers = FormParser, XMLParser, JSONParser

    def parse(self, request):
        " Parse request content "
        if request.method in ('POST', 'PUT', 'PATH'):
            content_type = self.determine_content(request)
            if content_type:
                split = content_type.split(';', 1)
                if len(split) > 1:
                    content_type = split[0]
                content_type = content_type.strip()

            parser = self.meta.parsers_dict.get(content_type, self.meta.default_parser)
            data = parser(self).parse(request)
            return dict() if isinstance(data, basestring) else data
        return dict()

    @staticmethod
    def determine_content(request):
        " Determine request content "

        if not request.META.get('CONTENT_LENGTH', None) and not request.META.get('TRANSFER_ENCODING', None):
            return None

        return request.META.get('CONTENT_TYPE', None)



from adrest.tests.utils import AdrestTestCase

from .api import API
from .models import Root, Child


class RPCTestCase(AdrestTestCase):
    api = API

    def setUp(self):
        self.root1 = Root.objects.create(name='test_root1')
        for i in xrange(10):
            Child.objects.create(root=self.root1, name='test_child1')

        self.root2 = Root.objects.create(name='test_root2')
        for i in xrange(10):
            Child.objects.create(root=self.root2, name='test_child2', odd=i % 2)

    def test_base(self):
        uri = self.reverse('test')
        self.assertEqual(uri, '/rpc/test/')

        response = self.get_resource('test')
        self.assertContains(response, 'true')

    def test_rpc_resource(self):
        uri = self.reverse('rpc')
        self.assertEqual(uri, '/rpc/rpc')

        response = self.get_resource('rpc')
        self.assertContains(response, 'Payload', status_code=402)

        response = self.rpc(dict(
            method='blablabla'
        ))
        self.assertContains(response, 'Wrong method', status_code=402)

        response = self.rpc(dict(
            method='bla.bla'
        ))
        self.assertContains(response, 'Invalid', status_code=402)

        response = self.rpc(dict(
            method='test.bla'
        ))
        self.assertContains(response, 'Invalid', status_code=402)

    def test_rpc_get(self):
        response = self.rpc(dict(
            method='test.get'
        ))
        self.assertContains(response, 'true')

        response = self.rpc(dict(
            method='root.get',
            headers=dict(Authorization=111)
        ))
        self.assertContains(response, 'test_root')

        response = self.rpc(dict(
            method='root-child.get',
            params=dict(root=self.root1.pk)
        ))
        self.assertContains(response, '"count": 10')
        self.assertContains(response, 'test_child1')
        self.assertNotContains(response, 'test_child2')

        response = self.rpc(dict(
            method='root-child.get',
            params=dict(root=self.root2.pk),
            data=dict(odd=1)
        ))
        self.assertContains(response, '"count": 5')

    def test_rpc_post(self):
        response = self.rpc(dict(
            method='root.post',
            data=dict(name='root3')
        ), key=111)
        self.assertContains(response, 'root3')

        response = self.rpc(dict(
            method='root-child.post',
            params=dict(root=self.root1.pk),
            data=dict(name='child3')
        ), key=111)
        self.assertContains(response, 'child3')

        child = Child.objects.get(name='child3')
        self.assertEqual(child.root, self.root1)

        response = self.rpc(dict(
            method='root-child.post',
            params=dict(root=self.root1.pk),
            data=dict(name='child4', root=self.root2.pk)
        ), key=111)
        self.assertContains(response, 'child4')

        child = Child.objects.get(name='child4')
        self.assertEqual(child.root, self.root1)

    def test_rpc_put(self):
        response = self.rpc(dict(
            method='root-child.put',
            params=dict(root=self.root1.pk, child=self.root1.child_set.all()[0].pk),
            data=dict(name='New name')
        ))
        self.assertContains(response, 'New name')
        child = self.root1.child_set.all()[0]
        self.assertEqual(child.name, 'New name')

    def test_rpc_jsonp(self):
        response = self.rpc(dict(
            method='test.get',
        ), callback='test1234', headers=dict(HTTP_ACCEPT='text/javascript'))
        self.assertEqual(response.get('Content-type'), 'text/javascript')
        self.assertContains(response, 'test1234')

        response = self.rpc(dict(
            method='test.get',
        ), callback='jsonp', headers=dict(HTTP_ACCEPT='text/javascript'))
        self.assertContains(response, 'jsonp')

        response = self.rpc(dict(
            method='test.get',
            callback='parseJSON'
        ), headers=dict(HTTP_ACCEPT='text/javascript'))
        self.assertContains(response, 'parseJSON')

        response = self.rpc(dict(
            method='test.get',
            callback='parseJSON'
        ), callback='Other', headers=dict(HTTP_ACCEPT='text/javascript'))
        self.assertContains(response, 'parseJSON')

        response = self.rpc(dict(
            method='test.get',
            data=dict(error=True)
        ), headers=dict(HTTP_ACCEPT='text/javascript'))
        self.assertEqual(response.get('Content-type'), 'text/javascript')
        self.assertContains(response, 'callback("Custom error")', status_code=400)



from django.http import QueryDict
from django.utils import simplejson

from ..utils.emitter import JSONPEmitter, JSONEmitter
from ..utils.exceptions import HttpError
from ..utils.status import HTTP_402_PAYMENT_REQUIRED
from ..views import ResourceView


class RPCResource(ResourceView):
    " Auto generated RPC. "

    url_regex = r'^rpc$'
    emitters = JSONEmitter, JSONPEmitter
    separator = '.'

    def __init__(self, *args, **kwargs):
        self.target_resource = None
        super(RPCResource, self).__init__(*args, **kwargs)

    def get(self, request, **resources):
        try:
            payload = request.GET.get('payload')
            payload = simplejson.loads(payload)
            assert payload, "Payload not found."

            method = payload['method']
            assert method and self.separator in method, "Wrong method name: %s." % method

            resource, method = method.split(self.separator, 1)
            resource = self.api.resources.get(resource)
            assert resource and hasattr(resource, method), "Wrong resource: %s.%s" % (resource, method)

            data = QueryDict('', mutable=True)
            data.update(payload.get('data', dict()))
            data['callback'] = payload.get('callback') or request.GET.get('callback') or request.GET.get('jsonp') or 'callback'

            for h, v in payload.get('headers', dict()).iteritems():
                request.META["HTTP_%s" % h.upper().replace('-', '_')] = v

            request.POST = request.PUT = request.GET = data
            delattr(request, '_request')

            request.method = method.upper()

        except AssertionError, e:
            raise HttpError('Invalid RPC Call. %s' % e, status=HTTP_402_PAYMENT_REQUIRED)

        except (ValueError, KeyError, TypeError):
            raise HttpError('Invalid RPC Payload.', status=HTTP_402_PAYMENT_REQUIRED)

        self.target_resource = resource
        resource = resource.as_view(api=self.api)
        return resource(request, _emit_=False, **payload.get("params", dict()))

    def get_name(self):
        if self.target_resource:
            return self.target_resource.meta.name
        return self.meta.name


# pymode:lint_ignore=E1103



from django.forms.models import ModelChoiceField
from django.utils.encoding import smart_unicode

from ..settings import MAP_TEMPLATE
from ..utils.auth import AnonimousAuthenticator
from ..utils.emitter import HTMLTemplateEmitter, JSONEmitter
from ..views import ResourceView


class MapResource(ResourceView):
    " Simple Api Map. "

    log = False
    emitters = HTMLTemplateEmitter, JSONEmitter
    authenticators = AnonimousAuthenticator
    template = MAP_TEMPLATE

    url_regex = r'^map$'

    def get(self, *args, **Kwargs):
        selfmap = list(self.gen_apimap())
        return self.api.str_version, selfmap

    def gen_apimap(self):
        for url_name in sorted(self.api.resources.iterkeys()):
            resource = self.api.resources[url_name]
            info = dict(
                name=url_name,
                emitters=', '.join([e.media_type for e in resource.emitters]),
                doc=resource.__doc__,
                methods=resource.allowed_methods,
                fields=[]
            )
            if resource.model:
                info['resource'] = resource.model.__name__

            models = [p.model for p in resource.meta.parents if p.model]

            if resource.form and ('POST' in resource.allowed_methods or 'PUT' in resource.allowed_methods):
                info['fields'] += [
                    (name, dict(
                        required=f.required and f.initial is None,
                        label=f.label,
                        help=smart_unicode(f.help_text + ''))
                     )
                    for name, f in resource.form.base_fields.iteritems()
                    if not (isinstance(f, ModelChoiceField) and f.choices.queryset.model in models)
                ]

            for a in resource.authenticators:
                info['fields'] += a.get_fields()

            info['auth'] = set(
                a.__doc__ or 'Custom' for a in resource.authenticators)
            key = resource.meta.url_regex.replace("(?P", "").replace(
                "[^/]+)", "").replace("?:", "").replace("$", "").replace("^", "/")
            yield key, info



from datetime import datetime
from os import path as op
from time import mktime

from django.db.models.base import ModelBase, Model
from django.http import HttpResponse
from django.template import RequestContext, loader

from ..utils import UpdatedList
from .paginator import Paginator
from .response import SerializedHttpResponse
from .serializer import JSONSerializer, XMLSerializer
from .status import HTTP_200_OK


class EmitterMeta(type):
    " Preload format attribute. "
    def __new__(mcs, name, bases, params):
        cls = super(EmitterMeta, mcs).__new__(mcs, name, bases, params)
        if not cls.format and cls.media_type:
            cls.format = str(cls.media_type).split('/')[-1]
        return cls


class BaseEmitter(object):
    """ All emitters must extend this class, set the media_type attribute, and
        override the serialize() function.
    """
    __metaclass__ = EmitterMeta

    media_type = None
    format = None

    def __init__(self, resource, request=None, response=None):
        self.resource = resource
        self.request = request
        self.response = response
        if not isinstance(response, HttpResponse):
            self.response = SerializedHttpResponse(response, mimetype=self.media_type, status=HTTP_200_OK)

    def emit(self):
        if not isinstance(self.response, SerializedHttpResponse):
            return self.response

        self.response.content = self.serialize(self.response.response)
        self.response['Content-type'] = self.media_type
        return self.response

    @staticmethod
    def serialize(content):
        " Get content and return string. "
        return content


class TextEmitter(BaseEmitter):
    media_type = 'text/plain'

    @staticmethod
    def serialize(content):
        " Get content and return string. "
        return unicode(content)


class JSONEmitter(BaseEmitter):
    media_type = 'application/json'

    def serialize(self, content):
        worker = JSONSerializer(
            _fields=getattr(self.resource, 'fields', None),
            _include=getattr(self.resource, 'include', None),
            _exclude=getattr(self.resource, 'exclude', None),
            **getattr(self.resource, 'related', dict())
        )
        return worker.serialize(content)


class JSONPEmitter(JSONEmitter):
    media_type = 'text/javascript'

    def serialize(self, content):
        content = super(JSONPEmitter, self).serialize(content)

        callback = self.request.GET.get('callback', 'callback')
        return u'%s(%s)' % (callback, content)


class XMLEmitter(BaseEmitter):

    media_type = 'application/xml'
    xmldoc_tpl = '<?xml version="1.0" encoding="utf-8"?>\n<response success="%s" version="%s" timestamp="%s">%s</response>'

    def serialize(self, content):
        worker = XMLSerializer(
            _fields=getattr(self.resource, 'fields', None),
            _include=getattr(self.resource, 'include', None),
            _exclude=getattr(self.resource, 'exclude', None),
            **getattr(self.resource, 'related', dict())
        )
        return self.xmldoc_tpl % (
            'true' if self.response.status_code == HTTP_200_OK else 'false',
            self.resource.version,
            int(mktime(datetime.now().timetuple())),
            worker.serialize(content)
        )


class TemplateEmitter(BaseEmitter):
    " Serialize by django templates. "

    def serialize(self, content):
        if self.response.status_code != HTTP_200_OK:
            template_name = op.join('api', 'error.%s' % self.format)
        else:
            template_name = self.resource.template or self.get_template_path(content)

        template = loader.get_template(template_name)
        return template.render(RequestContext(self.request, dict(
            content=content,
            emitter=self,
            resource=self.resource)))

    def get_template_path(self, content=None):

        if isinstance(content, Paginator):
            return op.join('api', 'paginator.%s' % self.format)

        if isinstance(content, UpdatedList):
            return op.join('api', 'updated.%s' % self.format)

        app = ''
        name = self.resource.get_name()

        if not content:
            content = self.resource.model

        if isinstance(content, (Model, ModelBase)):
            app = content._meta.app_label
            name = content._meta.module_name

        basedir = self.resource.api.prefix if getattr(self.resource, 'api', None) else 'api'
        return op.join(
            basedir,
            self.resource.version,
            app,
            "%s.%s" % (name, self.format)
        )


class JSONTemplateEmitter(TemplateEmitter):
    " Template emitter with JSON media type. "
    media_type = 'application/json'


class JSONPTemplateEmitter(TemplateEmitter):
    " Template emitter with javascript media type. "
    media_type = 'text/javascript'
    format = 'json'

    def serialize(self, content):
        content = super(JSONPTemplateEmitter, self).serialize(content)
        callback = self.request.GET.get('callback', 'callback')
        return '%s(%s)' % (callback, content)


class HTMLTemplateEmitter(TemplateEmitter):
    " Template emitter with HTML media type. "
    media_type = 'text/html'


class XMLTemplateEmitter(TemplateEmitter):
    " Template emitter with XML media type. "

    media_type = 'application/xml'
    xmldoc_tpl = '<?xml version="1.0" encoding="utf-8"?>\n<response success="%s" version="%s" timestamp="%s">%s</response>'

    def serialize(self, content):
        return self.xmldoc_tpl % (
            'true' if self.response.status_code == HTTP_200_OK else 'false',
            self.resource.version,
            int(mktime(datetime.now().timetuple())),
            super(XMLTemplateEmitter, self).serialize(content)
        )


try:
    from bson import BSON

    class BSONEmitter(BaseEmitter):
        media_type = 'application/bson'

        @staticmethod
        def serialize(content):
            return BSON.encode(content)

except ImportError:
    pass



from .models import Book

from django.http import HttpResponse

from adrest.utils.emitter import JSONEmitter
from adrest.utils.exceptions import HttpError
from adrest.views import ResourceView


class AuthorResource(ResourceView):
    allowed_methods = 'GET', 'POST', 'PATCH'
    model = 'main.author'
    url_regex = '^owner/$'


class BookResource(ResourceView):
    allowed_methods = 'GET', 'post', 'pUt', 'DELETE'
    model = 'main.book'
    parent = AuthorResource


class BookPrefixResource(BookResource):
    prefix = 'test'


class ArticleResource(ResourceView):
    allowed_methods = 'GET', 'PUT', 'DELETE'
    model = 'main.article'
    parent = BookPrefixResource

    def put(self, request, **kwargs):
        assert False, "Assertion error"

    def delete(self, request, **kwargs):
        raise Exception("Some error")


class OtherResource(ResourceView):
    parent = BookResource
    url_prefix = 'other'

    def get(self, request, **kwargs):
        return True


class SomeOtherResource(ResourceView):
    parent = AuthorResource
    url_params = 'device',

    def get(self, request, **kwargs):
        return self.paginate(request, [1, 2, 3])


class CustomResource(ResourceView):
    allowed_methods = 'GET', 'POST'
    model = 'main.book'
    queryset = Book.objects.all()
    template = 'main/custom.xml'

    def get(self, request, **kwargs):
        return list(self.queryset)

    def post(self, request, **resources):
        try:
            request.data['test'] = 123
        except TypeError, e:
            raise HttpError(dict(error=str(e)), status=400, emitter=JSONEmitter)


class DummyResource(ResourceView):
    def get(self, request, **resources):
        return True


class BSONResource(ResourceView):

    allowed_methods = 'GET', 'POST'

    COUNTER = 1

    def get(self, request, **resources):
        return dict(counter=self.COUNTER)

    def post(self, request, **resources):
        self.COUNTER += request.data.get('counter', 0)
        return dict(counter=self.COUNTER)


class CSVResource(ResourceView):
    allowed_methods = 'GET'

    def get(self, request, **resources):
        return HttpResponse('value'.encode("utf-16"), mimetype="text/csv")



import logging

from django.conf.urls.defaults import patterns
from django.dispatch import Signal

from .resources.map import MapResource
from .resources.rpc import RPCResource
from .views import ResourceView


LOG = logging.getLogger('adrest')


class Api(object):

    """ Implements a registry to tie together the various resources that make up
        an API.

        Especially useful for navigation, providing multiple versions of your API.

        :param version: Version info as string or iterable.
        :param api_map: Enable API map (true by default)
        :param api_prefix: API Url prefix ('api' by default)
        :param api_rpc: Enable automatic Json RPC (default false)

        Additional params will be putted in self resources.

    """
    def __init__(self, version=None, api_map=True, api_prefix='api', api_rpc=False, **params):
        self.version = self.str_version = version
        self.prefix = api_prefix
        self.params = params
        self.resources = dict()
        self.request_started = Signal()
        self.request_finished = Signal()

        if api_map:
            self.resources[MapResource.meta.url_name] = MapResource

        if api_rpc:
            self.resources[RPCResource.meta.url_name] = RPCResource

        if not isinstance(self.str_version, basestring):
            try:
                self.str_version = '.'.join(map(str, version or list()))
            except TypeError:
                self.str_version = str(version)

    def __str__(self):
        return self.str_version

    def register(self, resource, **params):
        """ Registers resource for the API.

            :param resource: Resource class for registration

            Additional params will be putted in the resource.
        """

        # Must be instance of ResourceView
        assert issubclass(resource, ResourceView), "%s not subclass of ResourceView" % resource

        # Fabric of resources
        params = dict(self.params, **params)
        if params:
            params['name'] = ''.join(bit for bit in resource.__name__.split('Resource') if bit).lower()
            params['__module__'] = '%s.%s' % (self.prefix, self.str_version.replace('.', '_'))
            params['__doc__'] = resource.__doc__
            resource = type('%s%s' % (resource.__name__, len(self.resources)), (resource,), params)

        if self.resources.get(resource.meta.url_name):
            LOG.warning("A new resource '%r' is replacing the existing record for '%s'" % (resource, self.resources.get(resource.url_name)))

        self.resources[resource.meta.url_name] = resource

    @property
    def urls(self):
        """ Provides URLconf details for the ``Api`` and all registered
            ``Resources`` beneath it.
        """
        urls = []

        for url_name in sorted(self.resources.keys()):

            resource = self.resources[url_name]
            urls.append(resource.as_url(
                api=self,
                name_prefix='-'.join((self.prefix, self.str_version)).strip('-'),
                url_prefix=self.str_version
            ))

        return patterns(self.prefix, *urls)



from ..utils import status
from ..utils.exceptions import HttpError
from ..utils.throttle import NullThrottle, AbstractThrottle


class ThrottleMeta(type):

    def __new__(mcs, name, bases, params):
        cls = super(ThrottleMeta, mcs).__new__(mcs, name, bases, params)
        assert issubclass(cls.throttle, AbstractThrottle), "'throttle' must be subclass of AbstractThrottle"
        return cls


class ThrottleMixin(object):

    __metaclass__ = ThrottleMeta

    throttle = NullThrottle

    def throttle_check(self):
        throttle = self.throttle()
        wait = throttle.should_be_throttled(self)
        if wait:
            raise HttpError("Throttled, wait %d seconds." % wait, status=status.HTTP_503_SERVICE_UNAVAILABLE)



from .resources import AuthorResource, BookPrefixResource, ArticleResource, SomeOtherResource, CustomResource, BSONResource, CSVResource
from adrest.api import Api
from adrest.utils.auth import AnonimousAuthenticator, AccessKeyAuthenticator, UserAuthenticator
from adrest.utils.emitter import XMLTemplateEmitter, JSONEmitter, BSONEmitter
from adrest.utils.parser import BSONParser
from adrest.utils.throttle import CacheThrottle


class CustomUserAuth(UserAuthenticator):
    username_fieldname = 'nickname'


API = Api(version=(1, 0, 0), emitters=(XMLTemplateEmitter, JSONEmitter), throttle=CacheThrottle, api_prefix='main')

API.register(AuthorResource,
             authenticators=(CustomUserAuth, AnonimousAuthenticator))
API.register(BookPrefixResource)
API.register(CustomResource)
API.register(ArticleResource, authenticators=AccessKeyAuthenticator)
API.register(SomeOtherResource, url_name='test', url_regex='test/mem/$')
API.register(BSONResource, parsers=(BSONParser,), emitters=(BSONEmitter,))
API.register(CSVResource)



from django.contrib import admin


try:
    from .models import Access

    class AccessAdmin(admin.ModelAdmin):
        list_display = 'status_code', 'uri', 'method', 'identifier', 'created_at', 'version'
        list_filter = 'method', 'version'
        search_fields = 'uri', 'identifier'
        date_hierarchy = 'created_at'

    admin.site.register(Access, AccessAdmin)

except ImportError:
    pass


try:
    from .models import AccessKey

    class AccessKeyAdmin(admin.ModelAdmin):
        list_display = 'key', 'user', 'created'
        search_fields = '=key', '=user'
        raw_id_fields = 'user',

    admin.site.register(AccessKey, AccessKeyAdmin)

except ImportError:
    pass


# pymode:lint_ignore=W0704



from .api import API
from milkman.dairy import milkman
from adrest.tests.utils import AdrestTestCase, TestCase


class SimpleTestCase(AdrestTestCase):
    api = API

    def test_base(self):
        self.assertEqual(self.api.str_version, '1.0b')

        uri = self.reverse('task')
        self.assertEqual(uri, '/simple/1.0b/task/')

        response = self.get_resource('task')
        self.assertContains(response, 'true')


class SerializerTest(TestCase):

    def setUp(self):
        for _ in range(1, 10):
            milkman.deliver('main.book')

    def test_simply(self):
        from adrest.utils.serializer import BaseSerializer
        from .models import Task

        data = [milkman.deliver(Task), milkman.deliver(Task), 28, 'string']
        serializer = BaseSerializer(_exclude='fake', _include='username', user=dict(
            _fields='email',
        ))
        self.assertEqual(serializer.options['_exclude'], set(['fake']))

        out = serializer.to_simple(data)
        self.assertEqual(out[0]['fields']['username'], data[0].user.username)

    def test_xml(self):
        from adrest.utils.serializer import XMLSerializer
        from ..main.models import Book
        worker = XMLSerializer()
        test = worker.serialize(Book.objects.all())
        self.assertTrue("author" in test)

    def test_json(self):
        from ..main.models import Author
        from adrest.utils.serializer import JSONSerializer
        authors = Author.objects.all()
        worker = JSONSerializer()
        test = worker.serialize(authors)
        self.assertTrue("main.author" in test)



from django.conf.urls.defaults import include, patterns
from rpc.api import API as rpc
from simple.api import API as simple

from .api import API as main
from .resources import DummyResource


urlpatterns = main.urls + patterns('',
    DummyResource.as_url(),
    (r'^simple/', include(simple.urls)),
    (r'^rpc/', include(rpc.urls)),
)
